# -*- coding: utf-8 -*-
# moebelgruppe.py

from grafikfenster import *
from moebel import Moebel

### ---------------------------------------------------
class MoebelGruppe(Moebel):
    """Klasse MoebelGruppe
    Realisiert Möbelgruppen nach dem Kompositummuster"""

    def __init__(self,
                 xPos=100,
                 yPos=200,
                 winkel=0,
                 farbe='red'):
        """Konstruktor mit vordefinierten Parametern
        für die Position, Winkel und Farbe;
        breite und tiefe sind zu Anfang 0,
        die Gruppe immer sichtbar"""
        self.__komponenten=[]
        Moebel.__init__(self, xPos, yPos, 0, 0, winkel, farbe, True)

    def FuegeHinzu(self, moebel):
        if moebel in self.__komponenten: return
        self.Loesche()
        self.__komponenten.append(moebel)
        if self.GibSichtbar(): moebel.Zeige()
        else: moebel.Verberge()
        self.SetzeKinderGruppenTransformation()
        self.AktualisiereAbmessungen()
        self.Zeichne()
        
    def Entferne(self, moebel):
        if not(moebel in self.__komponenten): return
        self.Loesche()
        moebel.Verberge()
        self.__komponenten.remove(moebel)
        moebel.SetzeGruppe(None)
        self.SetzeKinderGruppenTransformation()
        self.AktualisiereAbmessungen()
        self.Zeichne()
        
    def GibKindobjekt(self, position):
        """gibt für eine MoebelGruppe das Kindobjekt zur Position zurück"""
        if position<len(self.__komponenten):
            if position>=0:
                return self.__komponenten[position]
        return None
        
    def GibFigur(self):
        """definiert die zu zeichnende Figur"""
        path = self.GibZeichenPfad()
        for moebel in self.__komponenten:
            path.AddPath(moebel.GibFigur())
        return self.Transformiere(path)

    def SetzeGruppenTransformation(self, gruppenTransformation):
        '''setzt eine von der Gruppe vorgegebene Transformation und muss
        die geänderte Transformation auch an die Kinder weiterreichen'''
        super(MoebelGruppe, self).SetzeGruppenTransformation(gruppenTransformation)
        self.SetzeKinderGruppenTransformation()

    def SetzeKinderGruppenTransformation(self):
        for moebel in self.__komponenten:
            moebel.SetzeGruppenTransformation(self.GibGesamtTransformation())        
    
    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Loesche()
        super(MoebelGruppe, self).BewegeHorizontal(weite)
        self.SetzeKinderGruppenTransformation()
        self.Zeichne()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Loesche()
        super(MoebelGruppe, self).BewegeVertikal(weite)
        self.SetzeKinderGruppenTransformation()
        self.Zeichne()

    def Drehe(self, winkel):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Loesche()
        super(MoebelGruppe, self).Drehe(winkel)
        self.SetzeKinderGruppenTransformation()
        self.Zeichne()

    def Loesche(self):
        """entfernt die Gruppe aus der darstellung"""
        for moebel in self.__komponenten:
            moebel.Loesche()

    def Zeichne(self):
        """fuegt die Gruppe der Darstellung hinzu"""
        for moebel in self.__komponenten:
            moebel.Zeichne()

### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        return True

    def TestAnwendung1(self):
        """für eine Schrankwand"""
        global moebelgruppe
        from schrank import Schrank
        global schrank1, schrank2, schrank3
        schrank1=Schrank(0, 0, 60, 37, 0, 'black', False)
        schrank2=Schrank(60, 0, 60, 37, 0, 'red', False)
        schrank3=Schrank(120, 0, 60, 37, 0, 'green', False)
        moebelgruppe=MoebelGruppe(100,100, 0, 'black')
        moebelgruppe.Zeige()
        raw_input('FuegeHinzu(schrank1)-> weiter ?')
        moebelgruppe.FuegeHinzu(schrank1)
        raw_input('FuegeHinzu(schrank2)-> weiter ?')
        moebelgruppe.FuegeHinzu(schrank2)
        raw_input('FuegeHinzu(schrank3)-> weiter ?')
        moebelgruppe.FuegeHinzu(schrank3)
        raw_input('BewegeHorizontal(50)-> weiter ?')
        moebelgruppe.BewegeHorizontal(30)
        raw_input('BewegeHorizontal(50)-> weiter ?')
        moebelgruppe.BewegeVertikal(30)
        print( moebelgruppe.GibKindobjekt(1) )
        raw_input('Drehe(10)-> weiter ?')
        moebelgruppe.Drehe(10)
        raw_input('Drehe(-10)-> weiter ?')
        moebelgruppe.Drehe(-10)
        raw_input('Verberge()-> weiter ?')
        moebelgruppe.Verberge()
        
    def TestAnwendung2(self):
        """für eine Tischgruppe"""
        global moebelgruppe
        from stuhl import Stuhl
        from tisch import Tisch
        global stuhl1, stuhl2, stuhl3, stuhl4, tisch 
        stuhl1=Stuhl(0, 60, winkel=270)
        stuhl2=Stuhl(60, 0, winkel=0)
        stuhl3=Stuhl(120, 60, winkel=90)
        stuhl4=Stuhl(60, 120, winkel=180)
        tisch=Tisch(50, 50, 60, 60)
        moebelgruppe=MoebelGruppe(10,10, 0, 'black')
        moebelgruppe.Zeige()
        moebelgruppe.FuegeHinzu(stuhl1)
        raw_input('weiter ?')
        moebelgruppe.FuegeHinzu(stuhl2)
        raw_input('weiter ?')
        moebelgruppe.FuegeHinzu(stuhl3)
        raw_input('weiter ?')
        moebelgruppe.FuegeHinzu(stuhl4)
        raw_input('weiter ?')
        moebelgruppe.FuegeHinzu(tisch)
        raw_input('weiter ?')
        moebelgruppe.BewegeHorizontal(50)
        raw_input('weiter ?')
        moebelgruppe.BewegeVertikal(30)
        print( moebelgruppe.GibKindobjekt(1) )
        raw_input('weiter ?')
        moebelgruppe.Drehe(-10)
        raw_input('weiter ?')
        moebelgruppe.Verberge()
        
    def TestAnwendung3(self):
        """für eine Schrankwand mit einer Tischgruppe"""
        from schrank import Schrank
        global schrank1, schrank2, schrank3
        schrank1=Schrank(0, 0, 60, 37, 0, 'black', False)
        schrank2=Schrank(60, 0, 60, 37, 0, 'red', False)
        schrank3=Schrank(120, 0, 60, 37, 0, 'green', False)
        from stuhl import Stuhl
        from tisch import Tisch
        global stuhl1, stuhl2, stuhl3, stuhl4, tisch
        stuhl1=Stuhl(0, 60, winkel=270)
        stuhl2=Stuhl(60, 0, winkel=0)
        stuhl3=Stuhl(120, 60, winkel=90)
        stuhl4=Stuhl(60, 120, winkel=180)
        tisch=Tisch(50, 50, 60, 60)
        global moebelgruppe1, moebelgruppe2, moebelgruppe3
        moebelgruppe1=MoebelGruppe(30,10, 0, 'red')
        moebelgruppe2=MoebelGruppe(50,110, 0, 'green')
        moebelgruppe1.Zeige()
        moebelgruppe2.Zeige()
        moebelgruppe3=MoebelGruppe(10,10, 0, 'black')
        moebelgruppe3.Zeige()
        moebelgruppe1.FuegeHinzu(schrank1)
        moebelgruppe1.FuegeHinzu(schrank2)
        moebelgruppe1.FuegeHinzu(schrank3)
        raw_input('BewegeHorizontal(10)-> weiter ?')
        moebelgruppe1.BewegeHorizontal(10)
        raw_input('BewegeVertikal(10)-> weiter ?')
        moebelgruppe1.BewegeVertikal(10)
        raw_input('Drehe(30)-> weiter ?')
        moebelgruppe1.Drehe(30)
        raw_input('Drehe(-30)-> weiter ?')
        moebelgruppe1.Drehe(-30)
        raw_input('weiter ?')
        moebelgruppe1.Verberge()
        
        moebelgruppe2.FuegeHinzu(stuhl1)
        moebelgruppe2.FuegeHinzu(stuhl2)
        moebelgruppe2.FuegeHinzu(stuhl3)
        moebelgruppe2.FuegeHinzu(stuhl4)
        moebelgruppe2.FuegeHinzu(tisch)
        raw_input('BewegeHorizontal(10)-> weiter ?')
        moebelgruppe2.BewegeHorizontal(10)
        raw_input('BewegeVertikal(10)-> weiter ?')
        moebelgruppe2.BewegeVertikal(10)
        raw_input('Drehe(30)-> weiter ?')
        moebelgruppe2.Drehe(30)
        raw_input('Drehe(-30)-> weiter ?')
        moebelgruppe2.Drehe(-30)
        raw_input('Verberge()-> weiter ?')
        moebelgruppe2.Verberge()
        
        raw_input('moebelgruppe3.FuegeHinzu(moebelgruppe1) und (moebelgruppe2)-> weiter ?')
        moebelgruppe3.FuegeHinzu(moebelgruppe1)
        moebelgruppe3.FuegeHinzu(moebelgruppe2)
        raw_input('BewegeHorizontal(40)-> weiter ?')
        moebelgruppe3.BewegeHorizontal(40)
        raw_input('BewegeVertikal(-20)-> weiter ?')
        moebelgruppe3.BewegeVertikal(-20)
        raw_input('Drehe(60)-> weiter ?')
        moebelgruppe3.Drehe(60)
        raw_input('Drehe(-60)-> weiter ?')
        moebelgruppe3.Drehe(-60)
        raw_input('fertig!')
        moebelgruppe3.Verberge()
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
